import javax.swing.filechooser.FileFilter;
import java.io.File;

/**
 * MultipleChoiceQuestionGenerator.java 1.0 03/01/21
 * <br>
 * Provides a file filter for file choosers that only allows jpg, jpeg or gif files 
 * <br>
 * Author       : James McMahon
 * <br>
 * Created      : 1/02/2003
 * <br> 
 * Last Updated : 1/02/2003 
 */
 
public class ImageFileFilter extends FileFilter {
 	
 	/**
 	 * Only allows jpg, jpeg and gif files to be selected as well as directories
 	 * @return true If the file is allowed
 	 * @return false If the file is not allowed
 	 */
 	public boolean accept(File file) {
 		boolean accept = file.isDirectory();
 		
 		if(!accept) {
 			String suffix = getSuffix(file);
 			
 			if(suffix != null) {
 				accept = suffix.equals("jpg");
 				if (!accept) {//not a jpg file but may be a jpeg one
 					accept = suffix.equals("jpeg");
 					if(!accept) {//not jpg, jpeg but may be gif
 						accept = suffix.equals("gif");
 					}
 				}
 			}
 		}
 		return accept;
 	}
 	
 	/**
 	 * Returns the description of this file filter for the file chooser
 	 * @return String Image Files (*.jpg, *.jpeg, *.gif)
 	 */
 	public String getDescription() {
 		return "Image Files (*.jpg, *.jpeg, *.gif)";
 	}
 	
 	/**
 	 * returns the suffix of the fle
 	 * @param file The file you want the suffix of
 	 * @return String The filenames suffix
 	 */
 	private String getSuffix(File file){
 		
 		String s = file.getPath(), suffix = null;
 		
 		int i = s.lastIndexOf('.');
 		
 		if(i > 0 && i <s.length() - 1) {
 			suffix = s.substring(i+1).toLowerCase();	
 		}
 		return suffix;
 	}
}