 import javax.swing.*;
 import java.awt.*; 
 import java.io.*;
 
 /**
  * MultipleChoiceQuestionGenerator.java 1.0 03/01/21
  * <br>
  * Preview Panel that is used in the JFileChooser
 * <br>
 * Author       : James McMahon
 * <br>
 * Created      : 1/02/2003
 * <br> 
 * Last Updated : 1/02/2003 
 */
 
 public class ImagePreviewer extends JLabel {
    
    /**
     * The icon used for displaying the image
     */
 	private ImageIcon icon;
 
 	/** 
 	 * Sets up the image previewer label with a scaled version of the file
 	 *<br>
 	 * The boolean warning indicates if a warning about file sizes should be shown or not
 	 * 
 	 * @param file    The file that is to be previewed
 	 * @param warning If this is true then file size warnings will be popped up otherwise they won't 
 	 */
 	public void configure(File file, boolean warning) {
 		Dimension size = getSize();
 		Insets insets = getInsets();

		if (file != null) {
 			ImageIcon image = new ImageIcon(file.getPath());
 	
 			setIcon(icon = new ImageIcon(image.getImage().getScaledInstance(
 											size.width - insets.left - insets.right,
 											size.height - insets.top - insets.bottom,
 											Image.SCALE_SMOOTH)));
 			setToolTipText(file.getPath());

 			if (((image.getIconHeight() > 200) ||
 				(image.getIconWidth() > 200)) && 
 				warning) {
 				
 				Object[] message = new Object[5];
 				
 				if (file.length() >= 50000) {//if bigger than 50 Kbytes
 					message[0] = "File "+file.getName()+" is very big and could cause problems in saving and viewing the questionsets";
 					message[1] = "";
 					message[2] = "When saving details expect it to take a short amount of time.";
 					message[3] = "";
 					message[4] = "Consider changing the image size to 200 by 200 pixels";
 				}
 				else {
 					message[0] = "File "+file.getName()+" is very big and could cause problems in saving and viewing the questionsets";
 					message[1] = "";
 					message[2] = "";
 					message[3] = "Consider changing the image size to 200 by 200 pixels";
 					message[4] = "";
 				}
 				
 				JOptionPane.showMessageDialog(this,
				      message,
					  "Big image file",
					  JOptionPane.INFORMATION_MESSAGE);
 			}
 		}
 		else {//file is null so no path and no icon
			setIcon(null);		
			setToolTipText(null);
 		}				
 	}
 }