import java.awt.*;
import java.awt.event.*;
import javax.swing.*;
import java.io.*;
import java.util.*;

/**
 * MultipleChoiceQuestionGenerator.java 1.0 03/01/21
 * <br>
 * Main class, set's up a JFrame with a questiondetails panel and a save button
 * also implements an ActionListener for the save button
 * <br>
 * Author       : James McMahon
 * <br>
 * Created      : 1/02/2003
 * <br> 
 * Last Updated : 1/02/2003 
 */

public class MultipleChoiceQuestionGenerator extends JFrame implements ActionListener {
	
	/** 
	 * The panel that allows the user to enter data and have it saved
	 */
	private QuestionDetailsPanel questionDetailsPanel = null;
	
	/** 
	 * The save details button to let the user save the question set details to disk
	 */
	private JButton saveDetailsButton = new JButton("Save Question Details");
	
	/** 
	 * Allows the screen to be refreshed when clicked 
	 */
	private JButton refreshScreenButton = new JButton("Refresh Screen");
	
	/** 
	 * Stores the number of question sets that are wanted
	 */
	private int numberOfSetsWanted = 0;

	/** 
	 * The current questionset that the user is entering data for
	 */
	private int currentQuestionSetNum = 0;	

	/** 
	 * The number of questions the user wants to create
	 */
	private int numberOfQuestions = 0;
	
	/** 
	 * The current question that the user is entering data for
	 */
	private int currentQuestionNumber = 0;

	/** 
	 * Allows file access
	 */
	private FileAccess fileCreator = null;
	
	/** 
	 * The name of the question set
	 */ 
	private String name = "";
	
	/** 
	 * The scroll pane used for the details panel to be added to the screen
	 */
	private JScrollPane scrollPane = null;
	
	/**
	 * The panel that is used to display the menu options (save and refresh)
	 */
	private JPanel bottomMenu = null;
	
	/** 
	 * The root directory for the files to be saved in
	 */
	private String directoryName = "questionset";
	
	/**
	 * Sets up the application, and first finds out what the user wants to do;
	 * <br>
	 * The user can add to an existing question set or they can create an entirely new one
	 * <br>
	 * At the moment adding to a question set takes the form of adding additional sets and not adding
	 * questions to an existing set
	 *
	 */
	public MultipleChoiceQuestionGenerator() {
		
		//First do some house keeping to make the screen pretty
		Dimension screenSize = new Dimension(Toolkit.getDefaultToolkit().getScreenSize());

		this.setSize(new Dimension(screenSize.width, screenSize.height - 50));
		
		this.setTitle("Question Generator");
		
		this.getContentPane().setLayout(new BorderLayout());
		
		//add the window listener
		this.addWindowListener(new WindowAdapter() {
			public void windowClosing(WindowEvent e) {
				dispose();
				System.exit(0);
			}
		});
		
		this.setVisible(true);
		
		//first of all ask the user if they want to add extra sets to a question set
		int userOption = JOptionPane.showConfirmDialog(this, 
													 "Would you like to add sets to an existing question set?", 
													 "Mode of Operation...", 
													 JOptionPane.YES_NO_OPTION);
													 												
		if (userOption == 0) {//got a yes response so user wants to add sets to a questionset 
		
			//first let the user select the directory that contains the questionset sets
			final JFileChooser fileChooser = new JFileChooser(System.getProperty("user.dir"));
		
			final JDialog chooserDialog = new JDialog((Frame)null, 
													"Choose the Question set start up file to use...", 
													false);
			
			chooserDialog.getContentPane().add(fileChooser);
			chooserDialog.setTitle("Choose the Question set start up file to add sets to");
			chooserDialog.pack();
			chooserDialog.setVisible(true);
		
			fileChooser.setFileFilter(new TextFileFilter());
			fileChooser.addActionListener(new ActionListener() {
					public void actionPerformed(ActionEvent ae) {
						File file = fileChooser.getSelectedFile();
											
							if ((ae.getActionCommand()).equals(JFileChooser.APPROVE_SELECTION)) {	

								StringTokenizer tok = new StringTokenizer((fileChooser.getCurrentDirectory()).toString(), System.getProperty("file.separator"));
			
								int numTokens = tok.countTokens();
								
								//don't want the first lot of th edirectory name (only the last sub-directory is wanted)
								for(int i=0; i < (numTokens-1); i++) {
									tok.nextToken();
								}
								
								//this is the directory that would otherwise be the root entered by user
								directoryName = tok.nextToken();
 		
 								//next initlise file access with this directory
 								fileCreator = new FileAccess(directoryName);
								
								//next parse the next set number for adding to the file
								//it should return -1 if the file isn't a valid init file
								int numSetsCurrently = fileCreator.getNumberOfSets(file);
								
								if (numSetsCurrently != -1) {//got a valid init file to add to
									
									//don't need this anymore
									chooserDialog.setVisible(false);
									
									//make these start at 1 and the right value for set number
									currentQuestionSetNum = numSetsCurrently+1;
									currentQuestionNumber = 1;
									
									//get the number of sets the user wants to create
									int numberOfSetsToAdd = getNumberOfQuestionSets("Please enter the number of question sets you wish to add");
			
									//need to set this because it is used to determine how many times
									//around the question details panel needs to be displayed
									numberOfSetsWanted = numberOfSetsToAdd + numSetsCurrently;
		
									//do this once outside so we can create a new questionsetdetails file	
									name = getQuestionSetName(currentQuestionSetNum, numberOfSetsWanted);
									numberOfQuestions = getNumberOfQuestions(name);	
									
									//want to add to an existing set, so we don't overwrite the file
									fileCreator.addQuestionSetDetails(name,currentQuestionSetNum);
								
									//next get the user to enter details!
									setUpQuestionDetailsPanel(name,currentQuestionSetNum,currentQuestionNumber,numberOfQuestions);
								}
								else {//the file wasn't valid so ask user to select another one
								 	JOptionPane.showMessageDialog((JFrame)null,
										  "You need to select a valid question set start up file",
										  "Not a valid init file!",
										  JOptionPane.ERROR_MESSAGE);
								}			
							}
							else {
								//user probably wants to quit so ask for confirmation
								int optionReply = JOptionPane.showConfirmDialog((JFrame)null, 
													 "Did you mean to quit? Otherwise please select the directory with the current questionset sets", 
													 "About to quit...", 
													 JOptionPane.YES_NO_OPTION);
								if(optionReply == JOptionPane.YES_OPTION) {
									JOptionPane.showMessageDialog((JFrame)null,
										  "Sorry to see you go so soon...hope you come back",
										  "Quitting",
										  JOptionPane.INFORMATION_MESSAGE);
									dispose();
									System.exit(0);	
								}
							}
					}
				});
				
		}
		else if (userOption == 1) {//got a no so the user wants to create a new questionset
		
			//first find out where the user wants to store all of the files (the main directory)
			directoryName = getDirectoryNameFromUser();
 		
 			//next initlise file access with this directory
 			fileCreator = new FileAccess(directoryName);
 		
			//get the number of sets the user wants to create
			numberOfSetsWanted = getNumberOfQuestionSets("Please enter the number of question sets you wish to create");
			
			//make these start at 1
			currentQuestionSetNum = 1;
			currentQuestionNumber = 1;
		
			//do this once outside so we can create a new questionsetdetails file	
			name = getQuestionSetName(currentQuestionSetNum, numberOfSetsWanted);
			numberOfQuestions = getNumberOfQuestions(name);	
		
			fileCreator.createQuestionSetDetails(name,currentQuestionSetNum);
		
			//next get the user to enter details!
			setUpQuestionDetailsPanel(name,currentQuestionSetNum,currentQuestionNumber,numberOfQuestions);
		
		}
		else {//not a yes or no so must have quit
			JOptionPane.showMessageDialog(this,
										  "Sorry to see you go so soon...hope you come back",
										  "Quitting",
										   JOptionPane.INFORMATION_MESSAGE);
			dispose();
			System.exit(0);			
		}
	}
	
	/**
	 * Takes the user to the question details panel that results in them being asked to complete the
	 * details of each question in a question set for each question set 
	 * <br>
	 * The parameters are used both in displaying user feedback and in creating the file structures on disk
	 * 
	 * @param setName The name of the set the details are for
	 * @param currentQSNum The current question set number that the details relate to
	 * @param currentQN The current question number of the current question set that the details are for
	 * @param numOfQuestions The total number of questions that are needed for this question set to be completed
	 */
	private void setUpQuestionDetailsPanel(String setName,int currentQSNum,int currentQN,int numOfQuestions){
	
		//name, currentSetNumber, currentQuestionNumber, totalQuestionNumber
		questionDetailsPanel = new QuestionDetailsPanel(setName,currentQSNum,currentQN,numOfQuestions);
		
		scrollPane = new JScrollPane(questionDetailsPanel);
		bottomMenu = new JPanel(new FlowLayout());
		
		saveDetailsButton.addActionListener(this);
		saveDetailsButton.setToolTipText("Save Question Data");
		
		refreshScreenButton.addActionListener(this);
		refreshScreenButton.setToolTipText("Redraw the Screen");

		bottomMenu.add(saveDetailsButton);
		bottomMenu.add(refreshScreenButton);
		
		this.getContentPane().add(scrollPane, BorderLayout.CENTER);
		this.getContentPane().add(bottomMenu, BorderLayout.SOUTH);
		this.validate();
		this.repaint();
	}

	/** 
	 * Returns the number of question sets that the user wants to create
	 *
	 * @param text The text of the dialog the user is given
	 * @return int The number of questionsets the user wants to create
	 */
	private int getNumberOfQuestionSets(String text) {
		int value = -1;
		String input = null;
		
		while (value <= 0) {//not got a valid number so wait for one
			try {
				input = JOptionPane.showInputDialog(text);
				value = Integer.parseInt(input);
			}
			catch (NumberFormatException nfe){
				if (input == null) {//user tried to quit
					JOptionPane.showMessageDialog(this,
												  "You cannot quit at this time!",
												  "You tried to quit",
												   JOptionPane.ERROR_MESSAGE);	
			  	}
			 	else {
					JOptionPane.showMessageDialog(this,
											 	  "You need to input a number greater than 0",
												  "NumberFormat Error",
												  JOptionPane.ERROR_MESSAGE);
				}
			}
			
			//show an error relating to invalid number
			if ((value<=0) & (input != null)){
				JOptionPane.showMessageDialog(this,
											  "The number needs to be positive!",
											  "Invalid Negative Value",
											  JOptionPane.ERROR_MESSAGE);
			}
		}
		return value;
	}
	
	/** 
	 * Returns the name the user wants the questionset to be called
	 *
	 * @param currentSet The current set that is to be named
	 * @param questionSetNum The total number of question sets that are to be created
	 * @return String The name of the current question set gotten from the user
	 */
	private String getQuestionSetName(int currentSet, int questionSetNum){
		String name = "";
		
		while (name == null || name.equals("")) {
			name = JOptionPane.showInputDialog("Please enter the name of question set "+currentSet+" of "+questionSetNum);	
			if (name == null || name.equals("")) {
				if (name == null) {
					JOptionPane.showMessageDialog(this,
												  "You cannot quit at this time!",
												  "You tried to quit",
												   JOptionPane.ERROR_MESSAGE);	
				}
				else {
				JOptionPane.showMessageDialog(this,
											  "The name cannot be left blank!",
											  "Invalid Name Value",
											  JOptionPane.ERROR_MESSAGE);						
				}
			}
		}
		return name;
	}
	
	/** 
	 * Returns the name from the user that they want for the root directory.
	 *
	 * @return String The directory the user wants to store the questionsets in
	 */
	private String getDirectoryNameFromUser(){
		String name = "";
		boolean directoryExists = false;
		
		while (name == null 
				|| name.equals("") 
				|| (name.compareToIgnoreCase(" ") == 0)
				|| directoryExists) {
			
			name = JOptionPane.showInputDialog("Please enter root directory you want to store the files in");	
			

			if (name == null) {
				JOptionPane.showMessageDialog(this,
										  "You cannot quit at this time!",
										  "You tried to quit",
										   JOptionPane.ERROR_MESSAGE);	
			}
			else if (name.equals("")) {
				JOptionPane.showMessageDialog(this,
										  "The root directory name cannot be left blank!",
										  "Invalid Name Value",
										  JOptionPane.ERROR_MESSAGE);						
			}
			else if (name.compareToIgnoreCase(" ") == 0) {
				JOptionPane.showMessageDialog(this,
										  "The root directory name cannot contain blanks!",
										  "Invalid Name Value",
										  JOptionPane.ERROR_MESSAGE);						
			}
			else {//can find out if the name is already a directory
				File temp = new File(name);
				if (temp.isDirectory()){
					directoryExists = true;
					JOptionPane.showMessageDialog(this,
											  "That directory already exists! Choose a new name",
											  "Directory Exists",
											   JOptionPane.ERROR_MESSAGE);
				}
				else {//must be able to use the name as a directory
					directoryExists = false;
				}		
			}
		}
		return name;
	}
	
	/** 
	 * Returns the number of questions that the user wants in a question set 
	 * 
	 * @param questionSetName The name of the question set that the number of questions got relates to
	 * @return int The number of questions wanted in the questionset 
	 */
	private int getNumberOfQuestions(String questionSetName) {
		
		int value = -1;
		String input = null;
		
		while (value <= 0) {//not got a valid number so wait for one
			try {
				input = JOptionPane.showInputDialog("Please enter the number of questions for set "+questionSetName);
				value = Integer.parseInt(input);
			}
			catch (NumberFormatException nfe){
				if (input == null) {//user tried to quit
					JOptionPane.showMessageDialog(this,
												  "You cannot quit at this time!",
												  "You tried to quit",
												   JOptionPane.ERROR_MESSAGE);	
			  	}
			 	else {
					JOptionPane.showMessageDialog(this,
											 	  "You need to input a number greater than 0",
												  "NumberFormat Error",
												  JOptionPane.ERROR_MESSAGE);
				}
			}
			
			//show an error relating to invalid number
			if ((value<=0) & (input != null)){
				JOptionPane.showMessageDialog(this,
											  "The number needs to be positive!",
											  "Invalid Negative Value",
											  JOptionPane.ERROR_MESSAGE);
			}
		}
		return value;	
	}

	
	/** 
	 * The action listener which deals with the save button 
	 * @param ActionEvent The event generated when the save button is clicked
	 */
	public void actionPerformed(ActionEvent ae) {

		if (ae.getActionCommand().equals("Save Question Details")) {
			
			int validity = questionDetailsPanel.validateInput();
			
			//if valid then we want to ask for the rest of the details
			if (validity == 0) {
				
				//valid so save details and set up for the next lot if any more needs to be got
				questionDetailsPanel.saveDetails(directoryName);

				if (currentQuestionSetNum <= numberOfSetsWanted) {//then need to do another set at least
					
					//next question is this one
					currentQuestionNumber++;
					
					if (currentQuestionNumber <= numberOfQuestions) {//need to finish off this sets questions first
			
						//get rid of the old details panel from the screen
						this.getContentPane().remove(scrollPane);						
						
						
						//put a new questionsets panel on the screen for the next question	
						questionDetailsPanel = new QuestionDetailsPanel(name,currentQuestionSetNum, currentQuestionNumber ,numberOfQuestions);
						
						scrollPane = new JScrollPane(questionDetailsPanel);
						this.getContentPane().add(scrollPane);
						this.validate();
						this.repaint();
						this.setVisible(true);	
					}
					else {//no more questions needed for this set so go on to the next set
		
						//so update questionset number
						currentQuestionSetNum++;
					
						//may have already done the last question set so only do another if actually 
						//necessary
						if (currentQuestionSetNum <= numberOfSetsWanted) {		
							name = getQuestionSetName(currentQuestionSetNum, numberOfSetsWanted);
							numberOfQuestions = getNumberOfQuestions(name);
							//store the name of the questionset
							fileCreator.addQuestionSetDetails(name,currentQuestionSetNum);
						
							//get rid of the old details panel from the screen
							this.getContentPane().remove(scrollPane);						
						
							//put a new questionsets panel on the screen for the next question	
							currentQuestionNumber = 1;
							
							questionDetailsPanel = new QuestionDetailsPanel(name,currentQuestionSetNum, currentQuestionNumber ,numberOfQuestions);
						
							scrollPane = new JScrollPane(questionDetailsPanel);
							this.getContentPane().add(scrollPane);
							this.validate();
							this.repaint();
							this.setVisible(true);	
						}
						else {//finished all of the sets because there isn't another one to do!
							JOptionPane.showMessageDialog(this,
									      "Your question set data has now been saved!",
										  "Bye Bye",
										  JOptionPane.INFORMATION_MESSAGE);
							System.exit(0);//finshed creation so quit here
						}
					}
				}
			}	
			else if (validity == 1) {
					JOptionPane.showMessageDialog(this,
							      "All your options need images",
								  "Invalid Data",
								  JOptionPane.INFORMATION_MESSAGE);
			}
			else if (validity == 2) {
					JOptionPane.showMessageDialog(this,
							      "All your options require text or none can have any",
								  "Invalid Data",
								  JOptionPane.INFORMATION_MESSAGE);
			}
			else if (validity == 3) {
					JOptionPane.showMessageDialog(this,
							      "You need to provide textual options or image options at the very least!",
								  "Invalid Data",
								  JOptionPane.INFORMATION_MESSAGE);
			}
		}
		else if(ae.getActionCommand().equals("Refresh Screen")) {
			this.validate();
			this.repaint();
			questionDetailsPanel.refresh();
		}
	}
	
	/**
	 * The main method simply calls this constructot
	 */
	public static void main(String args[]) {
		System.out.println("Starting MultipleChoiceQuestionGenerator...");
		MultipleChoiceQuestionGenerator mainFrame = new MultipleChoiceQuestionGenerator();
	}
}