import java.awt.*;
import java.awt.event.*;
import java.io.*;
import javax.swing.*;

/**
 * MultipleChoiceQuestionGenerator.java 1.0 03/01/21
 * <br>
 * Panel which displays a screen the user can enter question data on to
 * <br>
 * Also provides a function for saving that data
 * <br>
 * Author       : James McMahon
 * <br>
 * Created      : 1/02/2003
 * <br> 
 * Last Updated : 1/02/2003 
 */

public class QuestionDetailsPanel extends JPanel {

	// The text panes that are used for entering the details
	// required for each question
	private JTextPane questionText      = new JTextPane();
	private JTextPane correctAnswerText = new JTextPane();
	private JTextPane falseAnswerText1  = new JTextPane();
	private JTextPane falseAnswerText2  = new JTextPane();
	private JTextPane falseAnswerText3  = new JTextPane();
	
	//The buttons used to allow the user to select images to insert
	//into a particular html file
	private JButton insertQuestionImage      = new JButton("Insert Question HTML Image");
	private JButton insertCorrectAnswerImage = new JButton("Insert Correct HTML Image");
	private JButton insertFalseAnswerImage1  = new JButton("Insert False 1 HTML Image");
	private JButton insertFalseAnswerImage2  = new JButton("Insert False 2 HTML Image");
	private JButton insertFalseAnswerImage3  = new JButton("Insert False 3 HTML Image");
	
	//buttons that allow the insertion of ImageIcons for each checkbox answer
	private JButton insertCorrectImageIcon = new JButton("Insert Correct Image Icon");
	private JButton insertFalseImageIcon1  = new JButton("Insert False Image Icon 1");
	private JButton insertFalseImageIcon2  = new JButton("Insert False Image Icon 2");
	private JButton insertFalseImageIcon3  = new JButton("Insert False Image Icon 3");
	
	//textfields that let the user insert the checkbox text for each checkbox
	private JTextField correctOption = new JTextField("", 20);
	private JTextField falseOption1  = new JTextField("", 20);
	private JTextField falseOption2  = new JTextField("", 20);
	private JTextField falseOption3  = new JTextField("", 20);
	
	//location of imageicons for checkboxes
	private File correctImageIcon = null;
	private File falseImageIcon1 = null;
	private File falseImageIcon2 = null;
	private File falseImageIcon3 = null;
	
	//location of URL's for images in the HTML pages
	private File questionImage = null;
	private File correctImage = null;
	private File falseImage1 = null;
	private File falseImage2 = null;
	private File falseImage3 = null;
		
	//values used in the creation of the files when the details are saved
	private String name = null;
	private int currentQuestionNumber    = 0;
	private int totalQuestionNumber      = 0;
	private int currentQuestionSetNumber = 0;
	
	//previewpanels for all of the imageicons that are available
	//the question html image if selected
	private ImagePreviewer questionHTMLImagePreviewer    = new ImagePreviewer();	
	private PreviewPanel   questionHTMLImagePreviewPanel = new PreviewPanel(questionHTMLImagePreviewer, "HTML Image");
	
	//the correct answer html page 
	private ImagePreviewer correctHTMLImagePreviewer    = new ImagePreviewer();	
	private PreviewPanel   correctHTMLImagePreviewPanel = new PreviewPanel(correctHTMLImagePreviewer, "HTML Image");
	
	//the first false answer page
	private ImagePreviewer false1HTMLImagePreviewer    = new ImagePreviewer();	
	private PreviewPanel   false1HTMLImagePreviewPanel = new PreviewPanel(false1HTMLImagePreviewer, "HTML Image");
	
	//the second false answer page
	private ImagePreviewer false2HTMLImagePreviewer    = new ImagePreviewer();	
	private PreviewPanel   false2HTMLImagePreviewPanel = new PreviewPanel(false2HTMLImagePreviewer, "HTML Image");
	
	//the third false answer page
	private ImagePreviewer false3HTMLImagePreviewer    = new ImagePreviewer();	
	private PreviewPanel   false3HTMLImagePreviewPanel = new PreviewPanel(false3HTMLImagePreviewer, "HTML Image");
	
	//the correct image icon
	private ImagePreviewer correctImageIconImagePreviewer    = new ImagePreviewer();	
	private PreviewPanel   correctImageIconImagePreviewPanel = new PreviewPanel(correctImageIconImagePreviewer, "Icon Image");
	
	//the first false image icon
	private ImagePreviewer false1ImageIconImagePreviewer    = new ImagePreviewer();	
	private PreviewPanel   false1ImageIconImagePreviewPanel = new PreviewPanel(false1ImageIconImagePreviewer, "Icon Image");
	
	//the second false image icon
	private ImagePreviewer false2ImageIconImagePreviewer    = new ImagePreviewer();	
	private PreviewPanel   false2ImageIconImagePreviewPanel = new PreviewPanel(false2ImageIconImagePreviewer, "Icon Image");
	
	//the second false image icon
	private ImagePreviewer false3ImageIconImagePreviewer    = new ImagePreviewer();	
	private PreviewPanel   false3ImageIconImagePreviewPanel = new PreviewPanel(false3ImageIconImagePreviewer, "Icon Image");
	
	/**
	 * Creates a panel where the user can enter the question details of a single 
	 * question based on the parameters passed in. 
	 * <br>
	 * N.B. The passed in parameters are also used in creating the file structures and files
	 * <br>
	 * @param name The name of the question set
	 * @param currentSetNumber The current set that this question will relate to
	 * @param currentQuestionNumber The current question number that this question is
	 * @param totalQuestionNumber The total number of questions that are wanted in this question set
	 */
	public QuestionDetailsPanel(String name, int currentSetNumber, int currentQuestionNumber, int totalQuestionNumber) {

		this.name                     = name;
		this.currentQuestionNumber    = currentQuestionNumber;
		this.totalQuestionNumber      = totalQuestionNumber;
		this.currentQuestionSetNumber = currentSetNumber;

		//holds the details for the question of the set
		JPanel details = new JPanel();
		
		details.setLayout(new GridLayout(5,1,5,5));
		details.setPreferredSize(new Dimension(850,725));
		details.setBorder(BorderFactory.createTitledBorder("Question "+currentQuestionNumber+" of "+totalQuestionNumber+" in Question Set: "+name));		
	
		QuestionDetailsPanelListener qdListener = new QuestionDetailsPanelListener(this);
	
		insertQuestionImage.addActionListener(qdListener);
	    insertCorrectAnswerImage.addActionListener(qdListener);
	    insertFalseAnswerImage1.addActionListener(qdListener);
	    insertFalseAnswerImage2.addActionListener(qdListener);
	    insertFalseAnswerImage3.addActionListener(qdListener);
		insertCorrectImageIcon.addActionListener(qdListener);
	    insertFalseImageIcon1.addActionListener(qdListener);
	    insertFalseImageIcon2.addActionListener(qdListener);
	    insertFalseImageIcon3.addActionListener(qdListener);
	
		details.add(questionSetup());
		details.add(correctAnsSetup());
		details.add(falseAns1Setup());
		details.add(falseAns2Setup());
		details.add(falseAns3Setup());
		
		this.add(details, BorderLayout.CENTER);
		this.setVisible(true);
	}	
		
	/** 
	 * Initlises the question panel
	 * 
	 * @return JPanel A panel with the buttons and image previewers 
	 *                setup on it for the question html page for this question
	 */
	private JPanel questionSetup() {
		
		//the panel that is returned
		JPanel question = new JPanel(new BorderLayout());
		question.setPreferredSize(new Dimension(250,300));
		question.setBorder(BorderFactory.createTitledBorder("Enter the Question Text here"));		
		
		//the internal panels that give a neater layout
		JPanel mainHolder  = new JPanel(new BorderLayout());
		JPanel imageHolder = new JPanel(new BorderLayout());
		
		mainHolder.add(new JScrollPane(questionText), BorderLayout.CENTER);
		mainHolder.add(insertQuestionImage, BorderLayout.EAST);
		
		imageHolder.add(questionHTMLImagePreviewPanel, BorderLayout.EAST);
		
		question.add(mainHolder, BorderLayout.CENTER);
		question.add(imageHolder, BorderLayout.EAST);
		
		return question;
	}
	
	/** 
	 * Initlises the correct answer panel
	 * 
	 * @return JPanel A panel with the buttons and image previewers 
	 *                setup on it for the correct answer of the question
	 */
	private JPanel correctAnsSetup() {
		
		//the panel that is returned
		JPanel correctAns = new JPanel(new BorderLayout());
		correctAns.setPreferredSize(new Dimension(250,300));
		correctAns.setBorder(BorderFactory.createTitledBorder("Enter the Correct Answer Text here"));		
			
		//the internal panels that give a neater layout
		JPanel bottomHolder = new JPanel(new FlowLayout());
		JPanel imageHolder  = new JPanel(new BorderLayout());
		JPanel mainHolder    = new JPanel(new BorderLayout());
		
		bottomHolder.add(new JLabel("Option Text :"));
		bottomHolder.add(correctOption);
		bottomHolder.add(insertCorrectImageIcon);
		
		imageHolder.add(correctHTMLImagePreviewPanel, BorderLayout.CENTER);
		imageHolder.add(correctImageIconImagePreviewPanel, BorderLayout.EAST);
		
		mainHolder.add(new JScrollPane(correctAnswerText), BorderLayout.CENTER);
		mainHolder.add(bottomHolder, BorderLayout.SOUTH);
		mainHolder.add(insertCorrectAnswerImage, BorderLayout.EAST);
		
		correctAns.add(mainHolder, BorderLayout.CENTER);
		correctAns.add(imageHolder, BorderLayout.EAST);
		
		return correctAns;
	}
	
	/** 
	 * Initlises the first false answer panel
	 * 
	 * @return JPanel A panel with the buttons and image previewers 
	 *                setup on it for the 1st false answer of the question
	 */
	private JPanel falseAns1Setup() {
		
		//the panel that is returned
		JPanel falseAns1 = new JPanel(new BorderLayout());
		falseAns1.setPreferredSize(new Dimension(250,300));
		falseAns1.setBorder(BorderFactory.createTitledBorder("Enter the 1st False Answer Text here"));		
				
		//the internal panels that give a neater layout
		JPanel bottomHolder = new JPanel(new FlowLayout());
		JPanel imageHolder  = new JPanel(new BorderLayout());
		JPanel mainHolder    = new JPanel(new BorderLayout());
		
		bottomHolder.add(new JLabel("Option Text :"));
		bottomHolder.add(falseOption1);
		bottomHolder.add(insertFalseImageIcon1);
		
		imageHolder.add(false1HTMLImagePreviewPanel, BorderLayout.CENTER);
		imageHolder.add(false1ImageIconImagePreviewPanel, BorderLayout.EAST);
		
		mainHolder.add(new JScrollPane(falseAnswerText1), BorderLayout.CENTER);
		mainHolder.add(bottomHolder, BorderLayout.SOUTH);
		mainHolder.add(insertFalseAnswerImage1, BorderLayout.EAST);
		
		falseAns1.add(mainHolder, BorderLayout.CENTER);
		falseAns1.add(imageHolder, BorderLayout.EAST);
		
		return falseAns1;
	}

	/** 
	 * Initlises the second false answer panel
	 * 
	 * @return JPanel A panel with the buttons and image previewers 
	 *                setup on it for the 2nd false answer of the question
	 */
	private JPanel falseAns2Setup() {
		
		//the panel that is returned
		JPanel falseAns2 = new JPanel(new BorderLayout());
		falseAns2.setPreferredSize(new Dimension(250,300));
		falseAns2.setBorder(BorderFactory.createTitledBorder("Enter the 2nd False Answer Text here"));		
				
		//the internal panels that give a neater layout
		JPanel bottomHolder = new JPanel(new FlowLayout());
		JPanel imageHolder  = new JPanel(new BorderLayout());
		JPanel mainHolder   = new JPanel(new BorderLayout());
		
		bottomHolder.add(new JLabel("Option Text :"));
		bottomHolder.add(falseOption2);
		bottomHolder.add(insertFalseImageIcon2);
		
		imageHolder.add(false2HTMLImagePreviewPanel, BorderLayout.CENTER);
		imageHolder.add(false2ImageIconImagePreviewPanel, BorderLayout.EAST);
		
		mainHolder.add(new JScrollPane(falseAnswerText2), BorderLayout.CENTER);
		mainHolder.add(bottomHolder, BorderLayout.SOUTH);
		mainHolder.add(insertFalseAnswerImage2, BorderLayout.EAST);
		
		falseAns2.add(mainHolder, BorderLayout.CENTER);
		falseAns2.add(imageHolder, BorderLayout.EAST);
		
		return falseAns2;
	}
	
	/** 
	 * Initlises the third false answer panel
	 * 
	 * @return JPanel A panel with the buttons and image previewers 
	 *                setup on it for the 3rd false answer of the question
	 */
	private JPanel falseAns3Setup() {
		
		//the panel that is returned
		JPanel falseAns3 = new JPanel(new BorderLayout());
		falseAns3.setPreferredSize(new Dimension(250,300));
		falseAns3.setBorder(BorderFactory.createTitledBorder("Enter the 3rd False Answer Text here"));		
				
		//the internal panels that give a neater layout
		JPanel bottomHolder = new JPanel(new FlowLayout());
		JPanel imageHolder  = new JPanel(new BorderLayout());
		JPanel mainHolder   = new JPanel(new BorderLayout());
		
		bottomHolder.add(new JLabel("Option Text :"));
		bottomHolder.add(falseOption3);
		bottomHolder.add(insertFalseImageIcon3);
		
		imageHolder.add(false3HTMLImagePreviewPanel, BorderLayout.CENTER);
		imageHolder.add(false3ImageIconImagePreviewPanel, BorderLayout.EAST);
		
		mainHolder.add(new JScrollPane(falseAnswerText3), BorderLayout.CENTER);
		mainHolder.add(bottomHolder, BorderLayout.SOUTH);
		mainHolder.add(insertFalseAnswerImage3, BorderLayout.EAST);
		
		falseAns3.add(mainHolder, BorderLayout.CENTER);
		falseAns3.add(imageHolder, BorderLayout.EAST);
		
		return falseAns3;
	}
	
	/** 
	 * Returns true if all text options are the empty strings
	 * 
	 * @return true All of the textual options are empty
	 * @return false Some or all of the textual options are not empty
 	 */
	private boolean allTextEmpty(){
		if ((correctOption.getText().equals("") && falseOption1.getText().equals("") && 
			 falseOption2.getText().equals("") && falseOption3.getText().equals(""))) {
			return true;	 	
		}
		return false;
	}
	
	/** 
	 * Returns true if all image options are null
	 * 
	 * @return true All the image icons are null
	 * @return false Not all of the image icons are null
	 */
	private boolean allImagesEmpty() {
		if ((correctImageIcon == null) && (falseImageIcon1 == null) && 
		    (falseImageIcon2 == null) && (falseImageIcon3 == null)) {
		   	return true;
		}
		return false;
	}
	
	/** 
	 * designed to be called when save data button is pressed it returns 0 if the data entered
	 * by the user can be considered valid 
	 * it returns 1 if there are image icons missing and returns 2 if option text is missing
	 * and returns 3 if both are missing
	 *
	 * @return 1 Indicates Required Image Icons are missing 
	 * @return 2 Indicates Required Option Text is missing
	 * @return 3 Indicates Required elements of both the Option Text and the Image Icons are missing
	 */ 
	protected int validateInput() {
		
		//if all no images or no text then kick us out
		if(allTextEmpty() && allImagesEmpty()) {
			return 3;	
		}		
		
		if ((correctOption.getText().equals("") && falseOption1.getText().equals("") && 
			falseOption2.getText().equals("") && falseOption3.getText().equals("")) ||
			(!correctOption.getText().equals("") && !falseOption1.getText().equals("") && 
			!falseOption2.getText().equals("") && !falseOption3.getText().equals(""))) {
			//all the same so don't kick us out but set valid data to true
		}
		else {
			return 2;//something missing in text options so kick out
		}
		
		
		if (((correctImageIcon == null) && (falseImageIcon1 == null) && 
		   (falseImageIcon2 == null ) && (falseImageIcon3 == null)) ||
		   ((correctImageIcon != null) && (falseImageIcon1 != null) && 
		   (falseImageIcon2 != null ) && (falseImageIcon3 != null))) { 
			//all same so can return valid data	
			return 0;
		}
		else {
			return 1;//something wrong with imageicons
		}
				
		//should never reach here
	}
	
	/** 
	 * Saves the questions details based on the information entered
     * <br>
     * Assumes information is all valid and requires the name of the root directory 
     * all files are to be saved in
     * <br>
	 * N.B. This is called from inside the SaveTask classes.
	 * <br>
	 * @param rootDirectory The location that all of the files are to be stored in
	 */
	protected void saveDetails(String rootDirectory){
		
		FileAccess fileCreator = new FileAccess(rootDirectory);
		  
		fileCreator.createDirectory("set"+currentQuestionSetNumber+"/q"+currentQuestionNumber);	
		
		if (questionImage == null) {
			System.out.println("Creating question file of set "+currentQuestionSetNumber+", question number : "+currentQuestionNumber+", without image");
			fileCreator.createQuestionFile(currentQuestionSetNumber, currentQuestionNumber, questionText.getText());
		} 
		else {//image was added so use the other constructor
			System.out.println("Creating question file of set "+currentQuestionSetNumber+", question number : "+currentQuestionNumber+", with image");
			fileCreator.createQuestionFile(currentQuestionSetNumber, currentQuestionNumber, questionText.getText(), questionImage);
		} 
		
		if (correctImage == null) {
			System.out.println("Creating correct answer file of set "+currentQuestionSetNumber+", question number : "+currentQuestionNumber+", without image");			
			fileCreator.createCorrectFile(currentQuestionSetNumber, currentQuestionNumber, correctAnswerText.getText());
		} 
		else {//image was added so use the other constructor
			System.out.println("Creating correct answer file of set "+currentQuestionSetNumber+", question number : "+currentQuestionNumber+", with image");
			fileCreator.createCorrectFile(currentQuestionSetNumber, currentQuestionNumber, correctAnswerText.getText(), correctImage);
		}
		
		if (falseImage1 == null) {
			System.out.println("Creating false answer file 1 of set "+currentQuestionSetNumber+", question number : "+currentQuestionNumber+", without image");
			fileCreator.createFalseFile(currentQuestionSetNumber, currentQuestionNumber, 1, falseAnswerText1.getText());
		} 
		else {//image was added so use the other constructor
			System.out.println("Creating false answer file 1 of set "+currentQuestionSetNumber+", question number : "+currentQuestionNumber+", with image");	
			fileCreator.createFalseFile(currentQuestionSetNumber, currentQuestionNumber, 1, falseAnswerText1.getText(), falseImage1);
		} 
	
		if (falseImage2 == null) {
			System.out.println("Creating false answer file 2 of set "+currentQuestionSetNumber+", question number : "+currentQuestionNumber+", without image");
			fileCreator.createFalseFile(currentQuestionSetNumber, currentQuestionNumber, 2, falseAnswerText2.getText());
		} 
		else {//image was added so use the other constructor
			System.out.println("Creating false answer file 2 of set "+currentQuestionSetNumber+", question number : "+currentQuestionNumber+", with image");
			fileCreator.createFalseFile(currentQuestionSetNumber, currentQuestionNumber, 2, falseAnswerText2.getText(), falseImage2);
		}
		
		if (falseImage3 == null) {
			System.out.println("Creating false answer file 3 of set "+currentQuestionSetNumber+", question number : "+currentQuestionNumber+", without image");
			fileCreator.createFalseFile(currentQuestionSetNumber, currentQuestionNumber, 3, falseAnswerText3.getText());
		} 
		else {//image was added so use the other constructor
			System.out.println("Creating false answer file 3 of set "+currentQuestionSetNumber+", question number : "+currentQuestionNumber+", with image");
			fileCreator.createFalseFile(currentQuestionSetNumber, currentQuestionNumber, 3, falseAnswerText3.getText(), falseImage3);
		}
	
		//created all of the html files so now need the questionset questions set-up file
		if ((correctImageIcon != null) && (!correctOption.getText().equals(""))) {//have both image and text
			System.out.println("Creating questionset file of set "+currentQuestionSetNumber+", question number : "+currentQuestionNumber+", with imageicons and text");			
			fileCreator.createQuestionSetFile(currentQuestionSetNumber, 
											  currentQuestionNumber, 
											  correctOption.getText(),
											  correctImageIcon,
											  falseOption1.getText(),
											  falseImageIcon1,
											  falseOption2.getText(),
											  falseImageIcon2,
											  falseOption3.getText(),
											  falseImageIcon3);
		}
		else if ((correctImageIcon != null) && (correctOption.getText().equals(""))) {//have text but no image
			System.out.println("Creating questionset file of set "+currentQuestionSetNumber+", question number : "+currentQuestionNumber+", without imageicons but with text");
			fileCreator.createQuestionSetFile(currentQuestionSetNumber, 
											  currentQuestionNumber, 
											  correctImageIcon,
											  falseImageIcon1,
											  falseImageIcon2,
											  falseImageIcon3);
		}
		else if ((correctImageIcon == null) && (!correctOption.getText().equals(""))) {//got image but no text
			System.out.println("Creating questionset file of set "+currentQuestionSetNumber+", question number : "+currentQuestionNumber+", with imageicons but without text");			
			fileCreator.createQuestionSetFile(currentQuestionSetNumber, 
											  currentQuestionNumber, 
											  correctOption.getText(),
											  falseOption1.getText(),
											  falseOption2.getText(),
											  falseOption3.getText());
		}
		System.out.println("Finshed creating the files of set "+currentQuestionSetNumber+", question number : "+currentQuestionNumber+".");		
	}
	
	//setter methods for all of the image urls
	
	//image icon files
	
	/**
	 * Loads the file for saving (when the user clicks save) and previewing on preview panel
	 * <br>
	 * Relates to the image for the correct image icon of the possible solution choices
	 * <br>
	 * @param loc The file that is to be saved and previewed
	 */	
	public void setCorrectImageIcon(File loc) {
		correctImageIcon = loc;
		correctImageIconImagePreviewer.configure(loc, false);
	}
	
	/**
	 * Loads the file for saving (when the user clicks save) and previewing on preview panel
	 * <br>
	 * Relates to the image for the false image icon one of the possible solution choices
	 * <br>
	 * @param loc The file that is to be saved and previewed
	 */		
	public void setFalseImageIcon1(File loc) {
		falseImageIcon1 = loc;
		false1ImageIconImagePreviewer.configure(loc, false);
	}
	
	/**
	 * Loads the file for saving (when the user clicks save) and previewing on preview panel
	 * <br>
	 * Relates to the image for the false image icon two of the possible solution choices
	 * <br>
	 * @param loc The file that is to be saved and previewed
	 */	
	public void setFalseImageIcon2(File loc) {
		falseImageIcon2 = loc;
		false2ImageIconImagePreviewer.configure(loc, false);
	}
	
	/**
	 * Loads the file for saving (when the user clicks save) and previewing on preview panel
	 * <br>
	 * Relates to the image for the false image icon three of the possible solution choices
	 * <br>
	 * @param loc The file that is to be saved and previewed
	 */
	public void setFalseImageIcon3(File loc) {
		falseImageIcon3 = loc;
		false3ImageIconImagePreviewer.configure(loc, false);
	}
	
	/**
	 * Loads the file for saving (when the user clicks save) and previewing on preview panel
	 * <br>
	 * Relates to the image for the actual question to be asked html page
	 * <br>
	 * @param loc The file that is to be saved and previewed
	 */
	public void setQuestionImage(File loc) {
		questionImage = loc;
		questionHTMLImagePreviewer.configure(loc, false);
	}
	
	/**
	 * Loads the file for saving (when the user clicks save) and previewing on preview panel
	 * <br>
	 * Relates to the image for the correct answer's html page
	 * <br>
	 * @param loc The file that is to be saved and previewed
	 */
	public void setCorrectImage(File loc) {
		correctImage = loc;
		correctHTMLImagePreviewer.configure(loc, false);
	}
	
	/**
	 * Loads the file for saving (when the user clicks save) and previewing on preview panel
	 * <br>
	 * Relates to the image for false answer 1's html page
	 * <br>
	 * @param loc The file that is to be saved and previewed
	 */
	public void setFalseImage1(File loc) {
		falseImage1 = loc;
		false1HTMLImagePreviewer.configure(loc, false);
	}
	
	/**
	 * Loads the file for saving (when the user clicks save) and previewing on preview panel
	 * <br>
	 * Relates to the image for false answer 2's html page
	 * <br>
	 * @param loc The file that is to be saved and previewed
	 */
	public void setFalseImage2(File loc) {
		falseImage2 = loc;
		false2HTMLImagePreviewer.configure(loc, false);
	}
	
	/**
	 * Loads the file for saving (when the user clicks save) and previewing on preview panel
	 * <br>
	 * Relates to the image for false answer 3's html page
	 * <br>
	 * @param loc The file that is to be saved and previewed
	 */
	public void setFalseImage3(File loc) {
		falseImage3 = loc;
		false3HTMLImagePreviewer.configure(loc, false);
	}
	
	/** 
	 * refresh all of the panels including the preview panels
	 */
	public void refresh(){
		this.validate();
		this.repaint();	

		correctImageIconImagePreviewPanel.refreshScreen();
		false1ImageIconImagePreviewPanel.refreshScreen();
		false2ImageIconImagePreviewPanel.refreshScreen();
		false3ImageIconImagePreviewPanel.refreshScreen();
		questionHTMLImagePreviewPanel.refreshScreen();
		correctHTMLImagePreviewPanel.refreshScreen();
		false1HTMLImagePreviewPanel.refreshScreen();
		false2HTMLImagePreviewPanel.refreshScreen();
		false3HTMLImagePreviewPanel.refreshScreen();
	}
	
	/**
	 * Returns the total size in bytes of all of the images that have been set for this question
	 * <br> 
	 * Also includes an approximate value of 5Kb for each of the question html, and 4 answer html files 
	 * <br><br>
	 * <b>This is redundant as it was originally intended as part of the JProgressBar that was not implementated</b>
	 * @return int The approximatet number of bytes that are to saved to disk in total
	 */
	public int getTotalBytesToSave() {
		
		int total = 5000;//the approximate size of the html files - could be improved to calculate it 
		
		if (questionImage != null) {
			total += questionImage.length();
		}
		
		if (correctImage != null) {
			total += correctImage.length();
		}
		
		if (falseImage1 != null) {
			total += falseImage1.length();
		}
		
		if (falseImage2 != null) {
			total += falseImage2.length();
		}
		
		if (falseImage3 != null) {
			total += falseImage3.length();
		}
		
		if (correctImageIcon != null) {
			total += correctImageIcon.length();
		}
		
		if (falseImageIcon1 != null) {
			total += falseImageIcon1.length();
		}
		
		if (falseImageIcon2 != null) {
			total += falseImageIcon2.length();
		}
		
		if (falseImageIcon3 != null) {
			total += falseImageIcon3.length();
		}
		return total;	
	}
}