import java.awt.*;
import java.awt.event.*;
import javax.swing.*;
import java.io.*;
import java.beans.*;

/**
 * MultipleChoiceQuestionGenerator.java 1.0 03/01/21
 * <br>
 * Panel which deals with button events in question detail 
 * <br>
 * Author       : James McMahon
 * <br>
 * Created      : 1/02/2003
 * <br> 
 * Last Updated : 1/02/2003 
 */

public class QuestionDetailsPanelListener implements ActionListener {
	
	/**
	 * The parent class used in dealing with the user's actions
	 */
	private QuestionDetailsPanel parent;
	
	/**
	 * Sets up the file chooser set to the user's directory to begin with
	 */
	private JFileChooser chooser = new JFileChooser(System.getProperty("user.dir"));
	
	/**
	 * The previewer for viewing the image files 
	 */
	private ImagePreviewer previewer = new ImagePreviewer();	

	/**
	 * The preview panel that the previewer resides on 
	 */
	private PreviewPanel previewPanel = new PreviewPanel(previewer, "Image Previewer");

	private JDialog dialog;
	
	//listeners which get added and removed as necessary from the chooser dialog
	private ActionListener listener1;
	private ActionListener listener2;
	private ActionListener listener3;
	private ActionListener listener4;
	private ActionListener listener5;
	private ActionListener listener6;
	private ActionListener listener7;
	private ActionListener listener8;
	private ActionListener listener9;
	
	
	/**
	 * Sets up the listener which needs access to the parent
	 * <br>
	 * Also sets the property change listener so that only valid image files can be selected
	 * and deals with adding them to the file chooser preview panel 
	 *
	 * @param qdParent The parent needed for setting image files etc
	 */
	public QuestionDetailsPanelListener(QuestionDetailsPanel qdParent) {
		parent = qdParent;

		chooser.setAccessory(previewPanel);
		chooser.setFileFilter(new ImageFileFilter());

		chooser.addPropertyChangeListener(
									new PropertyChangeListener() {
			public void propertyChange(PropertyChangeEvent e) {
				if(e.getPropertyName().equals(JFileChooser.SELECTED_FILE_CHANGED_PROPERTY)) {
										
				 	try{
						File file = (File)e.getNewValue();
						String s = file.getPath(), suffix = null;
						int i = s.lastIndexOf('.');
				
						if(i > 0 && i < s.length() - 1) {
							suffix = s.substring(i+1).toLowerCase();
						
							if (suffix.equals("gif") 
								|| suffix.equals("jpg") 
								|| suffix.equals("jpeg")) {
								previewer.configure(file, true);	
							}
						}
						else {//file is null (because it's a directory)
							previewer.configure(file, true);
						}
					} catch (NullPointerException npe) {
						//this will occur if the selection cannot be converted
						// to a usable file for the previewer 
						//catching the exception stops it being output to the screen
						//and removes any image in the previewer
						previewer.configure(null, true);
					}	
				}
			}
		});
	}
	
	/**
	 * Deals with all of the actions associated with the buttons on the main panel
	 * 
	 * @param ActionEvent The event that was generated by one of the buttons being pushed
	 */
	public void actionPerformed(ActionEvent ae) {

		if (ae.getActionCommand().equals("Insert Question HTML Image")) {
			
			dialog = new JDialog((Frame)null, "Choose A Question Page Image", false);
			
			dialog.getContentPane().add(chooser);
			dialog.setTitle("Choose A Question Page Image");
			dialog.pack();
			dialog.setVisible(true);
			
			chooser.addActionListener(listener1 = new ActionListener() {
				public void actionPerformed(ActionEvent ae) {
					File file = chooser.getSelectedFile();
					if ((ae.getActionCommand()).equals(JFileChooser.APPROVE_SELECTION)) {	
						parent.setQuestionImage(file);
					}
					else {
						//do nothing because the user cancelled selection
						parent.setQuestionImage(null);
					}
					chooser.removeActionListener(listener1);
					dialog.setVisible(false);
				}
			});
		}
		else if (ae.getActionCommand().equals("Insert Correct HTML Image")) { 
			
			dialog = new JDialog((Frame)null, "Choose Correct Answer Page Image", false);
			
			dialog.getContentPane().add(chooser);
			dialog.setTitle("Choose Correct Answer Page Image");
			dialog.pack();
			dialog.setVisible(true);
			
			chooser.addActionListener(listener2 = new ActionListener() {
				public void actionPerformed(ActionEvent ae) {
					File file = chooser.getSelectedFile();
					if ((ae.getActionCommand()).equals(JFileChooser.APPROVE_SELECTION)) {	
						parent.setCorrectImage(file);
					}
					else {
						//do nothing because the user cancelled selection
						parent.setCorrectImage(null);
					}
					chooser.removeActionListener(listener2);
					dialog.setVisible(false);
				}
			});	
		}	
		else if (ae.getActionCommand().equals("Insert False 1 HTML Image")) {
			
			dialog = new JDialog((Frame)null, "Choose False Answer Page 1 Image", false);
			
			dialog.getContentPane().add(chooser);
			dialog.setTitle("Choose False Answer Page 1 Image");
			dialog.pack();
			dialog.setVisible(true);
			
			chooser.addActionListener(listener3 = new ActionListener() {
				public void actionPerformed(ActionEvent ae) {
					File file = chooser.getSelectedFile();
					if ((ae.getActionCommand()).equals(JFileChooser.APPROVE_SELECTION)) {	
						parent.setFalseImage1(file);
					}
					else {
						//do nothing because the user cancelled selection
						parent.setFalseImage1(null);
					}
					chooser.removeActionListener(listener3);
					dialog.setVisible(false);
				}
			});
		
		}
		else if (ae.getActionCommand().equals("Insert False 2 HTML Image")) { 
			dialog = new JDialog((Frame)null, "Choose False Answer Page 2 Image", false);
			
			dialog.getContentPane().add(chooser);
			dialog.setTitle("Choose False Answer Page 2 Image");
			dialog.pack();
			dialog.setVisible(true);
			
			chooser.addActionListener(listener4 = new ActionListener() {
				public void actionPerformed(ActionEvent ae) {
					File file = chooser.getSelectedFile();
					if ((ae.getActionCommand()).equals(JFileChooser.APPROVE_SELECTION)) {	
						parent.setFalseImage2(file);
					}
					else {
						//do nothing because the user cancelled selection
						parent.setFalseImage2(null);
					}
					chooser.removeActionListener(listener4);
					dialog.setVisible(false);
				}
			});
		}
		else if (ae.getActionCommand().equals("Insert False 3 HTML Image")) { 
			dialog = new JDialog((Frame)null, "Choose False Answer Page 3 Image", false);
			
			dialog.getContentPane().add(chooser);
			dialog.setTitle("Choose False Answer Page 3 Image");
			dialog.pack();
			dialog.setVisible(true);
			
			chooser.addActionListener(listener5 = new ActionListener() {
				public void actionPerformed(ActionEvent ae) {
					File file = chooser.getSelectedFile();
					if ((ae.getActionCommand()).equals(JFileChooser.APPROVE_SELECTION)) {	
						parent.setFalseImage3(file);
					}
					else {
						//do nothing because the user cancelled selection
						parent.setFalseImage3(null);
					}
					chooser.removeActionListener(listener5);
					dialog.setVisible(false);
				}
			});
		}	
		else if (ae.getActionCommand().equals("Insert Correct Image Icon")) {
			dialog = new JDialog((Frame)null, "Choose Correct Option Image Icon", false);
			
			dialog.getContentPane().add(chooser);
			dialog.setTitle("Choose Correct Option Image Icon");
			dialog.pack();
			dialog.setVisible(true);
			
			chooser.addActionListener(listener6 = new ActionListener() {
				public void actionPerformed(ActionEvent ae) {
					File file = chooser.getSelectedFile();
					if ((ae.getActionCommand()).equals(JFileChooser.APPROVE_SELECTION)) {	
						parent.setCorrectImageIcon(file);
					}
					else {
						//do nothing because the user cancelled selection
						parent.setCorrectImageIcon(null);
					}
					chooser.removeActionListener(listener6);
					dialog.setVisible(false);
				}
			});
		}
		else if (ae.getActionCommand().equals("Insert False Image Icon 1")) {
			dialog = new JDialog((Frame)null, "Choose 1st False Option Image Icon", false);
			
			dialog.getContentPane().add(chooser);
			dialog.setTitle("Choose 1st False Option Image Icon");
			dialog.pack();
			dialog.setVisible(true);
			
			chooser.addActionListener(listener7 = new ActionListener() {
				public void actionPerformed(ActionEvent ae) {
					File file = chooser.getSelectedFile();
					if ((ae.getActionCommand()).equals(JFileChooser.APPROVE_SELECTION)) {	
						parent.setFalseImageIcon1(file);
					}
					else {
						//do nothing because the user cancelled selection
						parent.setFalseImageIcon1(null);
					}
					chooser.removeActionListener(listener7);
					dialog.setVisible(false);
				}
			});
		}
		else if (ae.getActionCommand().equals("Insert False Image Icon 2")) { 
			dialog = new JDialog((Frame)null, "Choose 2nd False Option Image Icon", false);
			
			dialog.getContentPane().add(chooser);
			dialog.setTitle("Choose 2nd False Option Image Icon");
			dialog.pack();
			dialog.setVisible(true);
			
			chooser.addActionListener(listener8 = new ActionListener() {
				public void actionPerformed(ActionEvent ae) {
					File file = chooser.getSelectedFile();
					if ((ae.getActionCommand()).equals(JFileChooser.APPROVE_SELECTION)) {	
						parent.setFalseImageIcon2(file);
					}
					else {
						//do nothing because the user cancelled selection
						parent.setFalseImageIcon2(null);
					}
					chooser.removeActionListener(listener8);
					dialog.setVisible(false);
				}
			});
		}
		else if (ae.getActionCommand().equals("Insert False Image Icon 3")) { 
			dialog = new JDialog((Frame)null, "Choose 3rd False Option Image Icon", false);
			
			dialog.getContentPane().add(chooser);
			dialog.setTitle("Choose 3rd False Option Image Icon");
			dialog.pack();
			dialog.setVisible(true);
			
			chooser.addActionListener(listener9 = new ActionListener() {
				public void actionPerformed(ActionEvent ae) {
					File file = chooser.getSelectedFile();
					if ((ae.getActionCommand()).equals(JFileChooser.APPROVE_SELECTION)) {	
						parent.setFalseImageIcon3(file);
					}
					else {
						//do nothing because the user cancelled selection
						parent.setFalseImageIcon3(null);
					}
					chooser.removeActionListener(listener9);
					dialog.setVisible(false);
				}
			});
		}	
	}
}