import javax.swing.filechooser.FileFilter;
import java.io.File;

/**
 * MultipleChoiceQuestionGenerator.java 1.0 03/01/21
 * <br>
 * Provides a file filter for file choosers that only allows directories
 * <br>
 * Author       : James McMahon
 * <br>
 * Created      : 1/02/2003
 * <br> 
 * Last Updated : 1/02/2003 
 */

public class TextFileFilter extends FileFilter {
 	
 	/** 
 	 * Only allows files that end in .txt or are directories to be accepted
 	 *
 	 * @param file The file that is to be accepted or rejected
 	 * @return false If the file is to be rejected 
 	 * @return true If the file is to be accepted
 	 */
 	public boolean accept(File file) {
 		boolean accept = file.isDirectory();
 		
 		if(!accept) {
 			String suffix = getSuffix(file);
 			
 			if(suffix != null) {
 				accept = suffix.equals("txt");
 			}
 		}
 		return accept;
 	}
 	
 	/**
 	 * A description of the file filter
 	 * @return String Text Files Only (*.txt)
 	 */
 	public String getDescription() {
 		return "Text Files Only (*.txt)";
 	}
 	
 	/**
 	 * Gets the suffix of the input file and returns it 
 	 * @param file The file that you want the suffix of
 	 * @return String The filename suffix
 	 */
 	private String getSuffix(File file){
 		
 		String s = file.getPath(), suffix = null;
 		
 		int i = s.lastIndexOf('.');
 		
 		if(i > 0 && i <s.length() - 1) {
 			suffix = s.substring(i+1).toLowerCase();	
 		}
 		return suffix;
 	}
}